/***************************************************************************
                          dlgxmlspecprocinstr.cpp  -  description
                             -------------------
    begin                : Ne ?ec 6 2003
    copyright            : (C) 2003 by The KXMLEditor Team
    email                : lvanek@eanet.cz
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "dlgxmlspecprocinstr.h"

#include <qstring.h>
#include <qcombobox.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qregexp.h>

#include <kdebug.h>
#include "dlgconfiguration.h"

DlgXMLSpecProcInstr::DlgXMLSpecProcInstr(QWidget *parent, const char *name )
  : DlgXMLSpecProcInstrBase(parent,name)
{

	DlgConfiguration::fillEncodings(m_pComboBoxEncoding);
  
  m_pLineEditVersion->setText("1.0");

  // signals and slots connections
  connect( m_pBtnOK, SIGNAL( clicked() ), this, SLOT( slotAccept() ) );
}

DlgXMLSpecProcInstr::~DlgXMLSpecProcInstr()
{
}

void DlgXMLSpecProcInstr::fillDialog( const QDomProcessingInstruction & domProcInstr )
{
  if(domProcInstr.target() != "xml")
  {
		kdError() << "DlgXMLProcInstr::fillDialog - use this dialog only for special XML proc. instr." << endl;
		return;
	}

  QString strData = domProcInstr.data();

  QString strVersion;
  QString strEncoding;

  int iStart, iEnd;

  // find version info
  if((iStart = strData.find("version", 0)) >= 0)
  {
    // info about encoding found;
    iStart += 7; // skip version

    // search " or ' after encoding
    if((iStart = strData.find(QRegExp("[\"']"), iStart)) > 0)
    {
      QChar ch = strData[iStart];
      iStart++; // skip ch
      if((iEnd = strData.find(ch, iStart)) > 0)
        strVersion = strData.mid(iStart, iEnd - iStart); 
    }
  }
  else
    strVersion = "1.0";
  
  // find encoding info
  if((iStart = strData.find("encoding", 0)) >= 0)
  {
    // info about encoding found;
    iStart += 8; // skip encoding

    // search " or ' after encoding
    if((iStart = strData.find(QRegExp("[\"']"), iStart)) > 0)
    {
      QChar ch = strData[iStart];
      iStart++; // skip ch
      if((iEnd = strData.find(ch, iStart)) > 0)
        strEncoding = strData.mid(iStart, iEnd - iStart);
    }
  }
  else
    strEncoding = "UTF-8";
  

  m_pLineEditVersion->setText(strVersion);
	m_pComboBoxEncoding->setCurrentText(strEncoding);
}

void DlgXMLSpecProcInstr::fillProcInstr( QDomProcessingInstruction & domProcInstr )
{
  QString strVersion = m_pLineEditVersion->text();
  QString strEncoding = m_pComboBoxEncoding->currentText();

  domProcInstr.setData("version='" + strVersion + "' encoding='" + strEncoding + "'");
}

int DlgXMLSpecProcInstr::exec( QDomProcessingInstruction & domProcInstr )
{
	fillDialog( domProcInstr );

	int iReturn = exec();
	if ( iReturn == Accepted )
		fillProcInstr( domProcInstr );

	return iReturn;
}

int DlgXMLSpecProcInstr::exec( QDomNode & domParentNode, QDomProcessingInstruction & domNewProcInstr )
{
	if ( domParentNode.isNull() )
	{
		kdError() << k_funcinfo << " The given parent object is empty." << endl;
		return Rejected;
	}
	else if ( ( ! domParentNode.isElement() ) && ( ! domParentNode.isDocument() ) )
	{
		kdError() << k_funcinfo << " The given parent object is of wrong type (" << domParentNode.nodeType() << ")." << endl;
		return Rejected;
	}

	int iReturn = exec();
	if ( iReturn == Accepted )
	{
		domNewProcInstr = domParentNode.ownerDocument().createProcessingInstruction( "xml", "version = '1.0' encoding = 'UTF-8'");
    fillProcInstr(domNewProcInstr);

		QDomNode domFirstChildNode = domParentNode.firstChild();
		if ( domFirstChildNode.isNull() )
			domParentNode.appendChild( domNewProcInstr ); // no childs yet -> simply append
		else
			domParentNode.insertBefore( domNewProcInstr, domFirstChildNode );
	}

	return iReturn;
}

int DlgXMLSpecProcInstr::exec()
{
	m_pBtnOK->setDefault(true);

	return DlgXMLSpecProcInstrBase::exec();
}


/** Called when user press OK button */
void DlgXMLSpecProcInstr::slotAccept()
{
  accept();
}


/*!
    Initializes content of dialog controls with specified values.
    	@param version XML file version
	@param encoding encoding type for the XML file
 */
void DlgXMLSpecProcInstr::fillDialog(const QString& version, const QString& encoding)
{

	m_pLineEditVersion->setText(version);
	m_pComboBoxEncoding->setCurrentText(encoding);
}


/*!
    Returns content of the dialog as an string of format: '"version = '...' encoding = '...' ".
 */
QString DlgXMLSpecProcInstr::getData()
{
    return 		QString("version = '")+m_pLineEditVersion->text()+
				"' encoding = '"+m_pComboBoxEncoding->currentText()+"' ";
}
