/***************************************************************************
                          kxmleditorpart.cpp  -  description
                             -------------------
    begin                : Wed Sep 19 2001
    copyright            : (C) 2001, 2002, 2003 by The KXMLEditor Team
    email                : OleBowle@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "kxmleditorpart.h"
#include "kxmleditorfactory.h"

#include "kxe_viewelement.h"
#include "kxe_treeviewitem.h"
#include "dlgconfiguration.h"
#include "dlgsearch.h"
#include "dlgchoosestring.h"
#include "dlgxmlelement.h"
#include "dlgxmlattribute.h"
#include "dlgxmlprocinstr.h"
#include "dlgxmlspecprocinstr.h"
#include "dlgxmlchardata.h"
#include "dlgxmlattachbase.h"

#include "actions.h"
#include "qdom_add.h"

#include <kinstance.h>
#include <kdebug.h>
#include <klocale.h>
#include <kaction.h>
#include <kstdaction.h>
#include <kpopupmenu.h>
#include <kiconloader.h>
#include <kmessagebox.h>
#include <ktar.h>
#include <kzip.h>
#include <kconfig.h>
#include <kurlrequester.h>
#include <qregexp.h>

#include <qtextcodec.h>
#include <qsplitter.h>
#include <qtabwidget.h>
#include <qtextedit.h>
#include <qcombobox.h>
#include <qfile.h>
#include <qtextstream.h>
#include <qclipboard.h>
#include <qdragobject.h>
#include <qapplication.h>
#include <qbuffer.h>
#include <qlabel.h>

#include "dcopiface_part_ro.h" // DCOP Iface

#define SCHEMA_NAMESPACE 		"http://www.w3.org/2001/XMLSchema-instance"
#define SCHEMA_ATTRIBUTE		"schemaLocation"
#define SCHEMA_ATTRIBUTE_XSI	"xsi:schemaLocation"

KXMLEditorPart::KXMLEditorPart( bool fReadWrite, QWidget * pParent, const char * pszName )
	: KParts::ReadWritePart(pParent,pszName),
	  m_pDoc( new QDomDocument() ),
	  m_pDlgSearch(0)
{
	//////////////////////////////
	// INIT PART
	//////////////////////////////

	setInstance(KXMLEditorFactory::instance());

	m_pBrowserExt = new KXMLEditorBrowserExtension( this, "KXMLEditorPart browser extension" );

	//////////////////////////////
	// CREATE ACTIONS
	//////////////////////////////

	// edit actions
	m_pActEditFind = KStdAction::find( this, SLOT(slotEditFind()), actionCollection(), "find" );
	m_pActEditFindNext = KStdAction::findNext( this, SLOT(slotEditFindNext()), actionCollection(), "findNext" );
	m_pActEditDeselect = new KAction( i18n("&Deselect Node"), 0, this,
	                                SLOT(slotEditDeselect()), actionCollection(), "deselect" );

	// view actions
	m_pActViewNodeUp = new KAction( i18n("To &Parent Node"), "up", 0, this,
	                                SLOT(slotViewNodeUp()), actionCollection(), "treeitem_up" );

	m_pActViewExpNode = new KToolBarPopupAction( i18n("&Expand Node"), "expand_node", CTRL+Key_Plus, this,
	                                             SLOT(slotViewExpNode()), actionCollection(), "treeitem_expand" );
	KPopupMenu * pMenuExpNode = m_pActViewExpNode->popupMenu();
	connect( pMenuExpNode, SIGNAL(activated(int)), this, SLOT(slotViewExpNode(int)) );
	for ( uint i = 1; i <= 8; i++ )
		pMenuExpNode->insertItem( i18n("Expand to level %1").arg(i), i-1 );

	m_pActViewColNode = new KToolBarPopupAction( i18n("&Collapse Node"), "collapse_node", CTRL+Key_Minus, this,
	                                             SLOT(slotViewColNode()), actionCollection(), "treeitem_collapse" );
	KPopupMenu * pMenuColNode = m_pActViewColNode->popupMenu();
	connect( pMenuColNode, SIGNAL(activated(int)), this, SLOT(slotViewColNode(int)) );
	for ( uint i = 0; i <= 7; i++ )
		pMenuColNode->insertItem( i18n("Collapse to level %1").arg(i), i );

	// bookmark actions
	m_pActBookmarksToggle = new KAction( i18n("&Toggle Bookmark"), "bookmark_add", CTRL+Key_B, this,
	                                   SLOT(slotBookmarksToggle()), actionCollection(), "bookmark_toggle" );
	m_pActBookmarksPrev = new KAction( i18n("&Previous Bookmark"), "bookmark_prev", SHIFT+Key_F5, this,
	                                   SLOT(slotBookmarksPrev()), actionCollection(), "bookmark_prev" );
	m_pActBookmarksNext = new KAction( i18n("&Next Bookmark"), "bookmark_next", Key_F5, this,
	                                   SLOT(slotBookmarksNext()), actionCollection(), "bookmark_next" );

	// settings actions
	new KAction( i18n("&Configure KXMLEditor..."), "configure", 0, this,
	             SLOT (slotConfigure()), actionCollection(), "configure" );

	// path toolbar
	m_pActPathCombo = new KXmlEditorComboAction( i18n("Path Bar"), 0, this,
	                                             SLOT(slotPathSelected(const QString &)), actionCollection(), "path_combo" );
	new KAction( i18n("Clear Path bar"), BarIcon("locationbar_erase", 16), 0, this,
	             SLOT(slotPathClear()), actionCollection(), "path_clear" );
	new KXmlEditorLabelAction( i18n("&Path "), actionCollection(), "path_label");

	if ( ! fReadWrite )
		setXMLFile( "kxmleditorpartBrowseUI.rc", true );
	else
	{
		m_pActXmlElementInsert = new KAction( i18n("&Insert Element..."), "xml_insert_element", CTRL+Key_E, this, SLOT(slotXmlElementInsert()), actionCollection(), "xml_ins_element" );
		m_pActXmlElementEdit = new KAction( i18n("&Edit Element..."), "xml_edit_element", 0, this, SLOT(slotXmlElementEdit()), actionCollection(), "xml_edit_element" );
		m_pActXmlElementDelete = new KAction( i18n("&Delete Element"), "xml_delete_element", 0, this, SLOT(slotXmlElementDelete()), actionCollection(), "xml_del_element" );

		m_pActXmlAttributesAdd = new KAction( i18n("Add &Attribute..."), "xml_insert_attribute", CTRL+Key_A, this, SLOT(slotXmlAttributesAdd()), actionCollection(), "xml_add_attribute" );
		m_pActXmlAttributesDel = new KAction( i18n("Delete all Attributes..."), 0, 0, this, SLOT(slotXmlAttributesDel()), actionCollection(), "xml_del_attributes" );
		m_pActXmlAttributeDel = new KAction( i18n("Delete Attribute"), "xml_delete_attribute", 0, this, SLOT(slotXmlAttributeDel()), actionCollection(), "xml_del_attribute" );

		m_pActXmlProcInstrInsert = new KAction( i18n("Insert Proc.Instruction..."), "xml_insert_procins", CTRL+Key_R, this, SLOT(slotXmlProcInstrInsert()), actionCollection(), "xml_ins_procins" );
		m_pActXmlProcInstrEdit = new KAction( i18n("Edit Proc.Instruction..."), "xml_edit_procins", 0, this, SLOT(slotXmlProcInstrEdit()), actionCollection(), "xml_edit_procins" );
		m_pActXmlProcInstrDelete = new KAction( i18n("Delete Proc.Instruction"), "xml_delete_procins", 0, this, SLOT(slotXmlProcInstrDelete()), actionCollection(), "xml_del_procins" );

    		m_pActXmlSpecProcInstrInsert = new KAction( i18n("Version and encoding..."), 0, CTRL+0, this, SLOT(slotXmlSpecProcInstrInsert()), actionCollection(), "xml_ins_spec_procins" );

		m_pActXmlAttachSchema = new KAction(i18n("Attach schema..."),0,0,this,SLOT(slotXmlAttachSchema()),actionCollection(),"xml_attach_schema");
		m_pActXmlDetachSchema = new KAction(i18n("Detach schema..."),0,0,this,SLOT(slotXmlDetachSchema()),actionCollection(),"xml_detach_schema");
		m_pActXmlAttachStylesheet = new KAction(i18n("Attach stylesheet..."),0,0,this,SLOT(slotXmlAttachStylesheet()),actionCollection(),"xml_attach_stylesheet");
		m_pActXmlDetachStylesheet = new KAction(i18n("Detach stylesheet..."),0,0,this,SLOT(slotXmlDetachStylesheet()),actionCollection(),"xml_detach_stylesheet");
		m_pActXmlAttachSchema->setEnabled(false);
		m_pActXmlDetachSchema->setEnabled(false);
		m_pActXmlDetachStylesheet->setEnabled(false);

		m_pActXmlCharDataInsert = new KAction( i18n("Insert Character Data..."), "xml_insert_contents", CTRL+Key_T, this, SLOT(slotXmlCharDataInsert()), actionCollection(), "xml_ins_contents" );
		m_pActXmlCharDataEdit = new KAction( i18n("Edit Character Data..."), "xml_edit_contents", 0, this, SLOT(slotXmlCharDataEdit()), actionCollection(), "xml_edit_contents" );
		m_pActXmlCharDataDelete = new KAction( i18n("Delete Character Data"), "xml_delete_contents", 0, this, SLOT(slotXmlCharDataDelete()), actionCollection(), "xml_del_contents" );

		m_pActXmlMoveNodeUp = new KAction( i18n("Move &Up"), "xml_move_item_up", CTRL+Key_U, this, SLOT(slotXmlMoveNodeUp()), actionCollection(), "xml_move_item_up" );
		m_pActXmlMoveNodeDown = new KAction( i18n("Move &Down"), "xml_move_item_down", CTRL+Key_D, this, SLOT(slotXmlMoveNodeDown()), actionCollection(), "xml_move_item_down" );

		setXMLFile( "kxmleditorpartEditUI.rc", true );
	}

	//////////////////////////////
	// CREATE WIDGETS
	//////////////////////////////

	QSplitter * pSplitter = new QSplitter( pParent, "KXMLEditorPart main widget (Splitter)" );
	pSplitter->setFocusPolicy( QWidget::StrongFocus );
	setWidget( pSplitter );

	// create configuration dialog -------------
	m_pDlgConfiguration = new DlgConfiguration(pSplitter);
	m_pDlgConfiguration->readConfiguration( instance()->config() );
	connect( m_pDlgConfiguration, SIGNAL(configurationChanged( const DlgConfiguration * const)), this, SLOT(slotConfigurationChanged()) );

	// create the tree view -------------------
	m_pViewTree = new KXE_TreeView( this, pSplitter, "KXMLEditorPart treeview" );

	connect( m_pViewTree, SIGNAL(sigSelectionCleared(bool)), this, SLOT(slotSelectionCleared(bool)) );
	connect( m_pViewTree, SIGNAL(sigSelectionChanged(const QDomElement &)), this, SLOT(slotSelectionChanged(const QDomElement &)) );
	connect( m_pViewTree, SIGNAL(sigSelectionChanged(const QDomCharacterData &)), this, SLOT(slotSelectionChanged(const QDomCharacterData &)) );
	connect( m_pViewTree, SIGNAL(sigSelectionChanged(const QDomProcessingInstruction &)), this, SLOT(slotSelectionChanged(const QDomProcessingInstruction &)) );
	connect( m_pViewTree, SIGNAL(sigContextMenuRequested(const QString&,const QPoint&)), this, SLOT(slotContextMenuRequested(const QString&,const QPoint&)) );
	connect( m_pViewTree, SIGNAL(itemRenamed(QListViewItem *)), this, SLOT(slotItemRenamedInplace(QListViewItem *)) );
	connect( m_pDlgConfiguration, SIGNAL(configurationChanged( const DlgConfiguration * const)), m_pViewTree, SLOT(slotConfigurationChanged( const DlgConfiguration * const)) );

	// create tab widget ----------------------
	m_pTabWidget = new QTabWidget( pSplitter, "KXMLEditorPart tabwidget");

	// create element view
	m_pViewElement = new KXE_ViewElement( m_pTabWidget, "KXMLEditorPart element view" );
	m_pTabWidget->addTab( m_pViewElement, g_iconElement, i18n("Element") );
	connect( m_pViewElement, SIGNAL(sigContextMenuRequested(const QString&,const QPoint&)), this, SLOT(slotContextMenuRequested(const QString&,const QPoint&)) );
	connect( m_pViewElement, SIGNAL(sigAttributeChangedInplace(const QDomAttr&)), this, SLOT(slotAttributeChangedInplace(const QDomAttr&)) );

	// create edit widget, that display XML contents
	m_pViewContents = new QTextEdit( m_pTabWidget, "KXMLEditorPart contents view" );
	m_pTabWidget->addTab( m_pViewContents, g_iconText, i18n("Contents") );
	m_pViewContents->setReadOnly( true );
	m_pViewContents->setWordWrap( QTextEdit::NoWrap );

	// create proc.instr. view
	m_pViewProcInstr = new QTextEdit( m_pTabWidget, "KXMLEditorPart proc.instr. view" );
	m_pTabWidget->addTab( m_pViewProcInstr, g_iconProcessingInstruction, i18n("Proc.Instruction") );
	m_pViewProcInstr->setReadOnly( true );
	m_pViewProcInstr->setWordWrap( QTextEdit::NoWrap );

	connect( this, SIGNAL(started(KIO::Job*)), this, SLOT(started()) );
	connect( this, SIGNAL(completed()), this, SLOT(completed()) );
	connect( this, SIGNAL(canceled(const QString &)), this, SLOT(canceled()) );

	//////////////////////////////
	// INIT BEGIN STATE
	//////////////////////////////

	// Disable actions
	m_pBrowserExt->emit enableAction( "copy", false );

	m_pActEditFindNext->setEnabled(false);
	m_pActEditDeselect->setEnabled(false);
	m_pActViewNodeUp->setEnabled(false);
	m_pActViewExpNode->setEnabled(false);
	m_pActViewColNode->setEnabled(false);
	m_pActBookmarksToggle->setEnabled(false);
	m_pActBookmarksPrev->setEnabled(false);
	m_pActBookmarksNext->setEnabled(false);

	if ( fReadWrite )
	{
		m_pBrowserExt->emit enableAction( "cut", false );
		m_pBrowserExt->emit enableAction( "paste", false );

		m_pActXmlElementInsert->setEnabled(true);
		m_pActXmlElementEdit->setEnabled(false);
		m_pActXmlElementDelete->setEnabled(false);
		m_pActXmlAttributesAdd->setEnabled(false);
		m_pActXmlAttributesDel->setEnabled(false);
		m_pActXmlAttributeDel->setEnabled(false);
		m_pActXmlProcInstrInsert->setEnabled(true);
		m_pActXmlProcInstrEdit->setEnabled(false);
		m_pActXmlProcInstrDelete->setEnabled(false);
		m_pActXmlCharDataInsert->setEnabled(false);
		m_pActXmlCharDataEdit->setEnabled(false);
		m_pActXmlCharDataDelete->setEnabled(false);
		m_pActXmlMoveNodeUp->setEnabled(false);
		m_pActXmlMoveNodeDown->setEnabled(false);
	}

	m_pTabWidget->setTabEnabled( m_pViewElement, false );
	m_pTabWidget->setTabEnabled( m_pViewContents, false );
	m_pTabWidget->setTabEnabled( m_pViewProcInstr, false );

	setReadWrite(fReadWrite);

	// use configuration
	m_pViewTree->slotConfigurationChanged( m_pDlgConfiguration );

	//////////////////////////////
	// INIT DCOP object (if any)
	//////////////////////////////
	//m_pDCOPIface = NULL;
	//m_pDCOPIface = new KXMLEditorPartIfaceReadOnly(this);
	m_pDCOPIface = new KXMLEditorPartIfaceReadWrite(this); // temporarly to test openUrl

	m_bDocIsCompressed = false;
}

KXMLEditorPart::~KXMLEditorPart()
{
	if (m_pDCOPIface)
		delete m_pDCOPIface;

	delete m_pDoc;
}


/////////////////////////////////////////////////////////////////////
//                   KPART FUNCTIONALITY
/////////////////////////////////////////////////////////////////////

bool KXMLEditorPart::openFile()
{
	if ( isModified() )
		kdError() << "KXMLEditorPart::openFile the current document is modified." << endl;

  QString strCompressedTarEntryName;

  // obtain file extension -----------------------------------------
	QString strExtension;

	QString strFileName = m_file;
	int iPos = strFileName.findRev('.');

	if(iPos > 0)
		{ strExtension = strFileName.mid(iPos + 1);
		}

	QString tmpfile;

  if(m_pDlgConfiguration->getListOfTarGzExtensions().contains(strExtension))
		{ /*KTarGz*/ KZip	tarGzFile(m_file); // new KOffice use KZip instead of KTarGz for storing files
			QFile fileTemporary("temporary.tmp");

			tarGzFile.open(IO_ReadOnly);
			fileTemporary.open(IO_WriteOnly);

			const KTarDirectory *root = tarGzFile.directory();
  		if(!root)
    		return false;

    	if(strCompressedTarEntryName.length() == 0)
    		{ DlgChooseString dlgChooseString(m_pViewTree, 0, i18n("Choose file"), i18n("File:"));
    		  dlgChooseString.m_pComboBox->insertItem("maindoc.xml");
    			dlgChooseString.m_pComboBox->insertItem("documentinfo.xml");
    			if(dlgChooseString.exec() != DlgChooseString::Accepted)
						{ return false;
						}
					m_strCompressedTarEntryName = dlgChooseString.m_strChoosedText;
				}
			 else
			 	{ m_strCompressedTarEntryName = strCompressedTarEntryName;
			 	}

			const /*KTarEntry*/ KArchiveEntry *entry = root->entry(m_strCompressedTarEntryName);

			if(entry && entry->isFile())
  			{ const /*KTarFile*/ KArchiveFile *pTarFile = static_cast <const /*KTarFile*/ KArchiveFile *> (entry);

    			QBuffer buffer(pTarFile->data());
    			buffer.open(IO_ReadOnly);

    			fileTemporary.writeBlock(buffer.buffer(), buffer.size());

  			}
  		else
  			m_strCompressedTarEntryName.truncate(0);

			tarGzFile.close();
			tmpfile = fileTemporary.name();
			fileTemporary.close();

			m_bDocIsCompressed = true;
		}
	else
		m_bDocIsCompressed = false;


	// ( 1.) parse the file and fill our document
	QFile file(m_bDocIsCompressed ? tmpfile : m_file);
	if(! file.open(IO_ReadOnly))
	{
		kdDebug() << "KXMLEditorPart::openFile: Can't open file." << endl;
		return false;
	}

  // auxiliary file for obtaining encoding info
  QFile fileAux(m_bDocIsCompressed ? tmpfile : m_file);
	if(! fileAux.open(IO_ReadOnly))
	{
		kdDebug() << "KXMLEditorPart::openFile: Can't open file." << endl;
		return false;
	}

	QTextStream txtStreamLocal( & file );

  // Lookup at XML document encoding -----------------------------------------------
  QTextStream txtStreamAux( & fileAux );
  QString strFirstLine = txtStreamAux.readLine();
  fileAux.close();
  int iStart, iEnd;
  if((iStart = strFirstLine.find("encoding", 0)) > 0)
  {
    QString strEncoding;
    // info about encoding found;
    iStart += 8; // skip encoding

    // search " or ' after encoding
    if((iStart = strFirstLine.find(QRegExp("[\"']"), iStart)) > 0)
    {
      QChar ch = strFirstLine[iStart];
      iStart++; // skip ch
      if((iEnd = strFirstLine.find(ch, iStart)) > 0)
      {
        strEncoding = strFirstLine.mid(iStart, iEnd - iStart);

        QTextCodec *pTextCodec = QTextCodec::codecForName(strEncoding);
        if(pTextCodec)
          txtStreamLocal.setCodec(pTextCodec);
        else
        {
          KMessageBox::sorry(0, i18n("Codec for encoding %1 not found ! Using locale encoding for load.").arg(strEncoding));
          txtStreamLocal.setEncoding(QTextStream::Locale);
        }
      }
    }
  }
  else
  {
    // XML documment dont have info about encoding, assume locale
    txtStreamLocal.setEncoding(QTextStream::Locale);
  }

  
  //--------------------------------------------------------------------------------
  QString strFileContents = txtStreamLocal.read();
	file.close();

  //-- Set string with XML to QDomDocument ------------------------------------------
	QString strErrorMsg;
  int iErrorLine, iErrorColumn;
	QDomDocument * pNewDoc = new QDomDocument; // first try with a new document

 	if( ! pNewDoc->setContent(strFileContents, true, &strErrorMsg, &iErrorLine, &iErrorColumn) )
 		{ kdDebug() << "KXMLEditorPart::openFile: Failed parsing the file." << endl;

			KMessageBox::error(0,
												 i18n("%1 in line %2, column %3").arg(strErrorMsg).arg(iErrorLine).arg(iErrorColumn),
												 i18n("Parsing error !"));

			delete pNewDoc; // remove the new document, because it's useless
			return false;
 		}

	delete m_pDoc;    // remove the former document
	m_pDoc = pNewDoc; // and take the new one

	// ( 2.) update the view
	m_pViewTree->updateClear();
	int i = m_pDoc->childNodes().length();
	while ( i > 0  )
	{
		i--;
		m_pViewTree->updateNodeCreated( m_pDoc->childNodes().item(i) );
	}

  m_pActPathCombo->slotClear();

  updateActions();
  return true;
}

bool KXMLEditorPart::saveFile()
{
	QString strXML;

	QTextStream streamXML(&strXML, IO_WriteOnly);
	m_pDoc->save(streamXML, 0);

  // decode string
	QString strEncoding;
  QTextCodec *pTextCodec;

  // find encoding info
  if(strXML.left(5) == "<?xml")
     { int iStart, iEnd;
       if((iStart = strXML.find("encoding", 0)) > 0)
       {
         // info about encoding found;
         iStart += 8; // skip encoding

         // search " or ' after encoding
         if((iStart = strXML.find(QRegExp("[\"']"), iStart)) > 0)
         {
           QChar ch = strXML[iStart];
           iStart++; // skip ch
           if((iEnd = strXML.find(ch, iStart)) > 0)
           {
             strEncoding = strXML.mid(iStart, iEnd - iStart);
           }
         }
       }
     }

	if(strEncoding.length() <= 0)
    pTextCodec = QTextCodec::codecForLocale(); // default
  else
    pTextCodec = QTextCodec::codecForName(strEncoding);
	 	
	if(pTextCodec == 0)
		{ if(KMessageBox::questionYesNo(0, i18n("Codec for encoding %1 not found ! Continue saving ?").arg(strEncoding)) != KMessageBox::Yes)
					return false;
		}
	    				
	QCString strDecoded;
	if(pTextCodec)
		{ strDecoded = pTextCodec->fromUnicode(strXML);
		}

  // save string to file
	if(!m_bDocIsCompressed)
	    { QFile file(m_file);
	      if(file.open(IO_WriteOnly) == true)
	        { file.writeBlock(strDecoded, strDecoded.length());
	  				file.flush();
	  				file.close();
	  			}
	  		else
					{ KMessageBox::error(0,
												 i18n("Can't create file %1").arg(m_file),
												 i18n("Write error !"));
					}
	  	}
	else
	  	{ /*KTarGz*/ KZip	tarGzFile(m_file); // New KOffice use KZip instead of KTarGz for storing files
	  		if(tarGzFile.open(IO_WriteOnly))
	  			{ tarGzFile.writeFile(m_strCompressedTarEntryName, "user", "group", strDecoded.length(), strDecoded);
	  				tarGzFile.close();
		  		}
		  	else
					{ KMessageBox::error(0,
												 i18n("Can't create archive %1").arg(m_file),
												 i18n("Write error !"));
					}
			}

  return true;
}

void KXMLEditorPart::setModified( bool fModified )
{
	KParts::ReadWritePart::setModified( fModified ); // base functionality

	static bool fAlreadyModified = false;

	if ( fAlreadyModified != fModified )
	{
		fAlreadyModified = fModified;

		QString szNewCaption = m_url.prettyURL();
		emit setWindowCaption( szNewCaption );
	}
}

void KXMLEditorPart::setReadWrite( bool fReadWrite )
{
	m_pViewTree->setReadWrite(fReadWrite);
	m_pViewElement->setReadWrite(fReadWrite);
//	m_pViewProcInstr->setReadOnly( ! fReadWrite );
//	m_pViewContents->setReadOnly( ! fReadWrite );

	KParts::ReadWritePart::setReadWrite( fReadWrite ); // base functionality
}

void KXMLEditorPart::started()
{
	kdDebug() << "KXMLEditorPart::started" << endl;
}
void KXMLEditorPart::completed()
{
	kdDebug() << "KXMLEditorPart::completed" << endl;
}
void KXMLEditorPart::canceled()
{
	kdDebug() << "KXMLEditorPart::canceled" << endl;
}


/////////////////////////////////////////////////////////////////////
//                     ACTION SLOTS
/////////////////////////////////////////////////////////////////////

void KXMLEditorPart::slotEditCut()
{
	kdDebug() << "KXMLEditor " << k_funcinfo << endl;

	if(! isReadWrite())
	{
		kdError() << "KXMLEditorPart::slotEditCut called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	
	if(pNode)
		{ // copy to clipboard
			slotEditCopy();

			// and delete it
			if( pNode->parentNode().removeChild( *pNode ).isNull() )
				kdError() << "KXMLEditorPart::slotEditCut error removing the selected node." << endl;
			else
				{ m_pViewTree->updateNodeDeleted(*pNode);
					setModified();
				}
		}
}

void KXMLEditorPart::slotEditCopy()
{
	kdDebug() << "KXMLEditor " << k_funcinfo << endl;

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	
	if(pNode)
		{	QTextDrag *pDrag = copyNode(m_pViewTree->getSelectedNode());
			if(pDrag)
				QApplication::clipboard()->setData(pDrag);
		}
	else
		kdError() << "KXMLEditorPart::slotEditCopy no element selected." << endl;
}

void KXMLEditorPart::slotEditPaste()
{
	kdDebug() << "KXMLEditor " << k_funcinfo << endl;

	if(!isReadWrite())
	{
		kdError() << "KXMLEditorPart::slotEditPaste called in readonly mode." << endl;
		return;
	}

	pasteNode(m_pViewTree->getSelectedNode(), QApplication::clipboard()->data());
}

void KXMLEditorPart::slotEditFind()
{
	emit setStatusBarText( i18n("Search in XML tree ...") );

	if ( ! m_pDlgSearch )
	{
		m_pDlgSearch = new DlgSearch( widget(), "search dialog", true );
	}

	if ( m_pDlgSearch->exec() == DlgSearch::Accepted )
		slotEditFindNext();

	m_pActEditFindNext->setEnabled(true);

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotEditFindNext()
{
	emit setStatusBarText( i18n("Search in XML tree ...") );

	if ( ! m_pDlgSearch )
	{
		kdDebug() << "KXMLEditorPart::slotEditFindNext implementation error - no search dialog" << endl;
		emit setStatusBarText( i18n("Ready.") );
		return;
	}

	// get node to start with (either the next node of the item selected at the tree view or documents root node)
	QDomNode node = ( (m_pViewTree->getSelectedNode()) && (! m_pViewTree->getSelectedNode()->isNull()) ) ? domTool_nextNode(* m_pViewTree->getSelectedNode()) : m_pDoc->documentElement();

	// start testing til the last node
	while( ! node.isNull() )
	{
		if ( domTool_match( node, m_pDlgSearch ) )
		{
			m_pViewTree->selectNode(node);
			emit setStatusBarText( i18n("Ready.") );
			return;
		}
		
		node = domTool_nextNode(node);
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlElementInsert()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlElementInsert called in readonly mode." << endl;
		return;
	}

	emit setStatusBarText( i18n("Inserting XML element into document...") );

	DlgXMLElement dlg( widget(), "XML element dialog" );
	int iDlgReturn = QDialog::Rejected;
	QDomElement domNewElement;

	if ( m_pDoc->documentElement().isNull() ) // the document doesn't
	{                                         // have a root element yet
		iDlgReturn = dlg.exec( * m_pDoc, domNewElement );
	}
	else  // the document seems to
	{     // have a root element
		QDomNode * pNode = m_pViewTree->getSelectedNode();

		if ( (pNode) && (pNode->isElement()) )
		{
			QDomElement domParentElement = pNode->toElement();
			iDlgReturn = dlg.exec( domParentElement, domNewElement );
		}
		else
			kdError() << "KXMLEditorPart::slotXmlElementInsert no element selected." << endl;
	}

	if ( iDlgReturn == QDialog::Accepted )
	{
		m_pViewTree->updateNodeCreated(domNewElement);
		setModified();
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlElementEdit()
{
	if ( ! isReadWrite() )
	{
	  kdError() << "KXMLEditorPart::slotXmlElementEdit called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( (!pNode) || (!pNode->isElement()) )
	{
		kdError() << "KXMLEditorPart::slotXmlElementEdit no node selected or selected node is no XML element." << endl;
		return;
	}

	emit setStatusBarText( i18n("Editing XML element...") );

	QDomElement domElement = pNode->toElement();

	DlgXMLElement dlg( widget(), "XML element dialog" );
	if ( dlg.exec( domElement ) == QDialog::Accepted )
	{
		m_pViewTree->updateNodeChanged(domElement);
		setModified();
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlElementDelete()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlElementDelete called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( (!pNode) || (!pNode->isElement()) )
	{
		kdError() << "KXMLEditorPart::slotXmlElementDelete no node selected or selected node is no XML element." << endl;
		return;
	}

	if ( pNode->parentNode().isNull() )
	{
		kdError() << "KXMLEditorPart::slotXmlElementDelete selected nodes parent node is empty." << endl;
		return;
	}

	emit setStatusBarText( i18n("Deleting XML element...") );

	if ( pNode->parentNode().removeChild( *pNode ).isNull() )
		kdError() << "KXMLEditorPart::slotXmlElementDelete error removing the selected node." << endl;
	else
	{
		m_pViewTree->updateNodeDeleted(*pNode);
		setModified();

    // if root element deleted, enable Insert Element
    if((m_pViewTree->firstChild() == 0) && (isReadWrite()))
      m_pActXmlElementInsert->setEnabled(true);
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlAttributesAdd()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlAttributesAdd called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( (!pNode) || (!pNode->isElement()) )
	{
		kdError() << "KXMLEditorPart::slotXmlAttributesAdd no node selected or selected node is no XML element." << endl;
		return;
	}

	emit setStatusBarText( i18n("Add attribute...") );

	DlgXMLAttribute dlg( widget(), "attribute dialog" );

	QDomElement domElement = pNode->toElement();
	if ( dlg.exec( domElement ) == QDialog::Accepted )
	{
		m_pViewTree->updateNodeChanged(domElement);
		m_pViewElement->slotChange(domElement);
		setModified();
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlAttributesDel()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlAttributesDel called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( (!pNode) || (!pNode->isElement()) )
	{
		kdError() << "KXMLEditorPart::slotXmlAttributesDel no node selected or selected node is no XML element." << endl;
		return;
	}

  if(KMessageBox::questionYesNo(0, i18n("Remove all attributes from selected node ?")) != KMessageBox::Yes)
    return;

	emit setStatusBarText( i18n("Delete all attributes...") );

	QDomElement domElement = pNode->toElement();
	QDomNamedNodeMap mapAttributes = domElement.attributes();
	uint nAttributes = mapAttributes.count();

	if ( nAttributes == 0 )
		return;

	for ( uint nRow = nAttributes; nRow > 0; nRow-- )
	{
		QDomNode node = mapAttributes.item(nRow-1);
		if ( node.isAttr() )
			domElement.removeAttributeNode(node.toAttr());
		else
			kdDebug() << "KXMLEditor " << k_funcinfo << " node is not an attribute (but should be)" << node.nodeName() << endl;
	}

	m_pViewTree->updateNodeChanged(domElement);
	m_pViewElement->slotChange(domElement);
	setModified();

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlAttributeDel()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlAttributeDel called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( (!pNode) || (!pNode->isElement()) )
	{
		kdError() << "KXMLEditorPart::slotXmlAttributeDel no node selected or selected node is no XML element." << endl;
		return;
	}

	emit setStatusBarText( i18n("Delete attribute...") );

	QDomElement domElement = pNode->toElement();
	QDomAttr domAttr = m_pViewElement->getSelectedAttribute();

	if ( ! domAttr.isNull() )
	{
		domElement.removeAttributeNode(domAttr);

		m_pViewTree->updateNodeChanged(domElement);
		m_pViewElement->slotChange(domElement);
		setModified();
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlProcInstrInsert()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlProcInstrInsert called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( (pNode) && (!pNode->isElement()) )
	{
		kdError() << k_funcinfo << " The selected node is no XML element." << endl;
		return;
	}
	else if ( ! pNode )  // no node selected -> the new node should be a direct child of the document
		pNode = m_pDoc;

	emit setStatusBarText( i18n("Inserting processing instruction into document...") );

	DlgXMLProcInstr dlg( widget(), "proc.instr. dialog" );
	QDomProcessingInstruction domProcInstr;

	if ( dlg.exec( *pNode, domProcInstr ) == QDialog::Accepted )
	{
		m_pViewTree->updateNodeCreated(domProcInstr);
		setModified();
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlProcInstrEdit()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlProcInstrEdit called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( (!pNode) || (!pNode->isProcessingInstruction()) )
	{
		kdError() << "KXMLEditorPart::slotXmlProcInstrEdit no node selected or selected node is no processing instruction." << endl;
		return;
	}

	emit setStatusBarText( i18n("Editing processing instruction...") );

	QDomProcessingInstruction domProcInstr = pNode->toProcessingInstruction();

  if(domProcInstr.target() == "xml")
    { KMessageBox::sorry(0, i18n("Pleasse use menu item XML -> Version and encoding for this processing instruction !"));
			return;
		}
  
	DlgXMLProcInstr dlg( widget(), "proc.instr. dialog" );

	if ( dlg.exec( domProcInstr ) == QDialog::Accepted )
	{
		m_pViewTree->updateNodeChanged(domProcInstr);
		m_pViewProcInstr->setText( domProcInstr.data() );
		setModified();
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlProcInstrDelete()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlProcInstrDelete called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( (!pNode) || (!pNode->isProcessingInstruction()) )
	{
		kdError() << "KXMLEditorPart::slotXmlProcInstrDelete no node selected or selected node is no processing instruction." << endl;
		return;
	}

	if ( pNode->parentNode().isNull() )
	{
		kdError() << "KXMLEditorPart::slotXmlProcInstrDelete selected nodes parent node is empty." << endl;
		return;
	}

	emit setStatusBarText( i18n("Deleting processing instruction...") );

	if ( pNode->parentNode().removeChild( *pNode ).isNull() )
		kdError() << "KXMLEditorPart::slotXmlProcInstrDelete error removing the selected node." << endl;
	else
	{
		m_pViewTree->updateNodeDeleted(*pNode);
		setModified();
	}

	emit setStatusBarText( i18n("Ready.") );
}


// Instert or edit special processing instruction <?xml ... ?>
void KXMLEditorPart::slotXmlSpecProcInstrInsert()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlSpecProcInstrInsert called in readonly mode." << endl;
		return;
	}

	emit setStatusBarText( i18n("Setting XML version and encoding of document...") );

	QDomNode * pNode = m_pViewTree->getSpecProcInstrNode("xml");
        DlgXMLSpecProcInstr dlg( widget() );

	if (pNode!=0)
		dlg.fillDialog(pNode->toProcessingInstruction());
	else
		dlg.fillDialog(m_pDlgConfiguration->getDefaultVersion(),m_pDlgConfiguration->getDefaultEncoding());

	if(dlg.exec())
	{
		setSpecProcInstr("xml",dlg.getData());
	}
}


void KXMLEditorPart::slotXmlCharDataInsert()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlCharDataInsert called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( (!pNode) || (!pNode->isElement()) )
	{
		kdError() << "KXMLEditorPart::slotXmlCharDataInsert no element selected." << endl;
		return;
	}

	emit setStatusBarText( i18n("Inserting character data into document...") );

	DlgXMLCharData dlg( widget(), "char.data dialog" );
	QDomCharacterData domCharData;
	QDomElement domParentElement = pNode->toElement();

	if ( dlg.exec( domParentElement, domCharData ) == QDialog::Accepted )
	{
		m_pViewTree->updateNodeCreated(domCharData);
		setModified();
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlCharDataEdit()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlCharDataEdit called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( (!pNode) || (!pNode->isCharacterData()) )
	{
		kdError() << "KXMLEditorPart::slotXmlCharDataEdit no node selected or selected node is no character data." << endl;
		return;
	}

	emit setStatusBarText( i18n("Editing character data...") );

	QDomCharacterData domCharData = pNode->toCharacterData();
	DlgXMLCharData dlg( widget(), "char.data dialog" );

	if ( dlg.exec( domCharData ) == QDialog::Accepted )
	{
		m_pViewTree->updateNodeChanged(domCharData);
		m_pViewContents->setText( domCharData.data() );
		setModified();
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlCharDataDelete()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlCharDataDelete called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( (!pNode) || (!pNode->isCharacterData()) )
	{
		kdError() << "KXMLEditorPart::slotXmlCharDataDelete no node selected or selected node is no character data." << endl;
		return;
	}

	if ( pNode->parentNode().isNull() )
	{
		kdError() << "KXMLEditorPart::slotXmlCharDataDelete selected nodes parent node is empty." << endl;
		return;
	}

	emit setStatusBarText( i18n("Deleting character data...") );

	if ( pNode->parentNode().removeChild( *pNode ).isNull() )
		kdError() << "KXMLEditorPart::slotXmlCharDataDelete error removing the selected node." << endl;
	else
	{
		m_pViewTree->updateNodeDeleted(*pNode);
		setModified();
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlMoveNodeUp()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlMoveNodeUp called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( ! pNode )
	{
		kdError() << "KXMLEditorPart::slotXmlMoveNodeUp no node selected." << endl;
		return;
	}

	QDomNode domParentNode = pNode->parentNode();
	if ( domParentNode.isNull() )
	{
		kdError() << "KXMLEditorPart::slotXmlMoveNodeUp selected nodes parent node is empty." << endl;
		return;
	}

	QDomNode domPrevSibling = pNode->previousSibling();
	if ( domPrevSibling.isNull() )
	{
		kdError() << "KXMLEditorPart::slotXmlMoveNodeUp selected node doesn't seem to have a previous sibling." << endl;
		return;
	}

	emit setStatusBarText( i18n("Moving node up...") );

	QDomNode domNode = domParentNode.removeChild( * pNode );
	if ( domNode.isNull() )
		kdError() << "KXMLEditorPart::slotXmlMoveNodeUp can't remove child node." << endl;
	else
	{
		domNode = domParentNode.insertBefore( domNode, domPrevSibling );
		if ( domNode.isNull() )
			kdError() << "KXMLEditorPart::slotXmlMoveNodeUp can't insert child node." << endl;
		else
		{
			m_pViewTree->updateNodeMoved(domNode);
			setModified();
		}
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotXmlMoveNodeDown()
{
	if ( ! isReadWrite() )
	{
		kdError() << "KXMLEditorPart::slotXmlMoveNodeDown called in readonly mode." << endl;
		return;
	}

	QDomNode * pNode = m_pViewTree->getSelectedNode();
	if ( ! pNode )
	{
		kdError() << "KXMLEditorPart::slotXmlMoveNodeDown no node selected." << endl;
		return;
	}

	QDomNode domParentNode = pNode->parentNode();
	if ( domParentNode.isNull() )
	{
		kdError() << "KXMLEditorPart::slotXmlMoveNodeDown selected nodes parent node is empty." << endl;
		return;
	}

	QDomNode domNextSibling = pNode->nextSibling();
	if ( domNextSibling.isNull() )
	{
		kdError() << "KXMLEditorPart::slotXmlMoveNodeDown selected node doesn't seem to have a next sibling." << endl;
		return;
	}

	emit setStatusBarText( i18n("Moving node down...") );

	QDomNode domNode = domParentNode.removeChild( * pNode );
	if ( domNode.isNull() )
		kdError() << "KXMLEditorPart::slotXmlMoveNodeDown can't remove child node." << endl;
	else
	{
		domNode = domParentNode.insertAfter( domNode, domNextSibling );
		if ( domNode.isNull() )
			kdError() << "KXMLEditorPart::slotXmlMoveNodeDown can't insert child node." << endl;
		else
		{
			m_pViewTree->updateNodeMoved(domNode);
			setModified();
		}
	}

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotBookmarksToggle()
{
	m_pViewTree->bookmarksToggle();
	m_pActBookmarksPrev->setEnabled(m_pViewTree->containsBookmarkedItems());
	m_pActBookmarksNext->setEnabled(m_pViewTree->containsBookmarkedItems());
}

void KXMLEditorPart::slotConfigure()
{
	emit setStatusBarText( i18n("Configure KXML Editor ...") );

	m_pDlgConfiguration->exec();

	emit setStatusBarText( i18n("Ready.") );
}

void KXMLEditorPart::slotPathSelected( const QString & strPath )
{
	QDomNode node = domTool_matchingNode( m_pDoc->documentElement(), strPath );
	if(node.isNull())
	  { // node don't exists, remove item from combo
      m_pActPathCombo->removeItem(strPath);
      m_pActPathCombo->slotClearEdit();
		  return;
	  }

	if(!m_pViewTree->selectNode(node))
		{ // node not found, remove item from combo
      m_pActPathCombo->removeItem(strPath);
      m_pActPathCombo->slotClearEdit();
    }
}

void KXMLEditorPart::slotPathClear()
{
	slotEditDeselect();
	m_pActPathCombo->slotFocusEdit();
}

void KXMLEditorPart::slotItemRenamedInplace( QListViewItem * pItem )
{
	KXE_TreeViewItem * pXMLItem = static_cast <KXE_TreeViewItem*> (pItem);

	if ( ! pXMLItem->xmlNode()->isElement() ) // check, if it really represents an XML element
	{
		kdFatal() << "KXMLEditorPart " << k_funcinfo << " the given item doesn't represent an XML element." << endl;
		return;
	}

	if ( pXMLItem->xmlNode()->toElement().nodeName() != pItem->text(0) ) // if really something was changed
	{
		if ( pXMLItem->xmlNode()->namespaceURI().isNull() )              // if the element is not "namespaced"
			pXMLItem->xmlNode()->toElement().setTagName( pItem->text(0) ); // set the entered text as tag name
		else
		{                                                                // else the element is "namespaced"
			int nPosColon = pItem->text(0).find(':');
			if ( nPosColon == -1 )                                           // if no namespace prefix was entered,
			{
				pXMLItem->xmlNode()->toElement().setPrefix( QString::null );   // clear the elements namespace prefix
				pXMLItem->xmlNode()->toElement().setTagName( pItem->text(0) ); // and set the entered text as its tag name
			}
			else                                                                            // otherwise split up the entered 
			{                                                                               // text by the first colon and 
				pXMLItem->xmlNode()->toElement().setPrefix( pItem->text(0).left(nPosColon) ); // use the 1st part as prefix and
				pXMLItem->xmlNode()->toElement().setTagName( pItem->text(0).right( pItem->text(0).length() - nPosColon - 1 ) ); // the 2nd one as local tag name
			}
		}

		setModified();

		if ( m_pViewTree->selectedItem() == pItem )                       // and if the item is still selected
		{
			m_pViewElement->slotChange( pXMLItem->xmlNode()->toElement() ); // update the element view and
			m_pActPathCombo->insertItem( domTool_getIconForNodeType(pXMLItem->xmlNode()->nodeType(), false),
			                             domTool_getPath(*pXMLItem->xmlNode()) ); // the path combo
		}
	}
}

void KXMLEditorPart::slotAttributeChangedInplace( const QDomAttr & domAttr )
{
	setModified();

	if ( *m_pViewTree->getSelectedNode() == domAttr.ownerElement() ) // if the corresponding element
	{                                                               // is still selected
		m_pViewElement->slotChange( domAttr.ownerElement() );
		m_pViewTree->updateNodeChanged( domAttr.ownerElement() );
	}
}

/////////////////////////////////////////////////////////////////////
//                     MISC FUNCTIONS
/////////////////////////////////////////////////////////////////////

void KXMLEditorPart::slotSelectionCleared(bool bRootElementExists)
{
	// Enable/Disable actions
	m_pBrowserExt->emit enableAction( "copy", false );

	m_pActEditDeselect->setEnabled(false);
	m_pActViewNodeUp->setEnabled(false);
	m_pActViewExpNode->setEnabled(false);
	m_pActViewColNode->setEnabled(false);
	m_pActBookmarksToggle->setEnabled(false);

	if ( isReadWrite() )
	{
		m_pBrowserExt->emit enableAction( "cut", false );
		m_pBrowserExt->emit enableAction( "paste", true );

    m_pActXmlElementInsert->setEnabled(!bRootElementExists);
		m_pActXmlElementEdit->setEnabled(false);
		m_pActXmlElementDelete->setEnabled(false);
		m_pActXmlAttributesAdd->setEnabled(false);
		m_pActXmlAttributesDel->setEnabled(false);
		m_pActXmlAttributeDel->setEnabled(false);
		m_pActXmlProcInstrInsert->setEnabled(true);
		m_pActXmlProcInstrEdit->setEnabled(false);
		m_pActXmlProcInstrDelete->setEnabled(false);
		m_pActXmlCharDataInsert->setEnabled(false);
		m_pActXmlCharDataEdit->setEnabled(false);
		m_pActXmlCharDataDelete->setEnabled(false);
		m_pActXmlMoveNodeUp->setEnabled(false);
		m_pActXmlMoveNodeDown->setEnabled(false);
	}

	m_pActPathCombo->slotClearEdit();

	// change views
	m_pViewContents->clear();

	m_pTabWidget->setTabEnabled( m_pViewElement, false );
	m_pTabWidget->setTabEnabled( m_pViewContents, false );
	m_pTabWidget->setTabEnabled( m_pViewProcInstr, false );

	m_pTabWidget->showPage( m_pViewContents );

	// change path combo
	m_pActPathCombo->slotClearEdit();
}

void KXMLEditorPart::slotSelectionChanged( const QDomElement & selectedNode)
{
	// Enable/Disable actions
	m_pBrowserExt->emit enableAction( "copy", true );

	m_pActEditDeselect->setEnabled(true);
	m_pActViewNodeUp->setEnabled( ! selectedNode.parentNode().isNull() ); // disable if it's a root item
	m_pActViewExpNode->setEnabled( ! selectedNode.firstChild().isNull() ); // no childs -> disable
	m_pActViewColNode->setEnabled( ! selectedNode.firstChild().isNull() ); // no childs -> disable
	m_pActBookmarksToggle->setEnabled(true);

	if ( isReadWrite() )
	{
		m_pBrowserExt->emit enableAction( "cut", true );
		m_pBrowserExt->emit enableAction( "paste", true );

		m_pActXmlElementInsert->setEnabled(true);
		m_pActXmlElementEdit->setEnabled(true);
		m_pActXmlElementDelete->setEnabled(true);
		m_pActXmlAttributesAdd->setEnabled(true);
		m_pActXmlAttributesDel->setEnabled( (selectedNode.attributes().count() != 0) );
		m_pActXmlAttributeDel->setEnabled(true);
		m_pActXmlProcInstrInsert->setEnabled(true);
		m_pActXmlProcInstrEdit->setEnabled(false);
		m_pActXmlProcInstrDelete->setEnabled(false);
		m_pActXmlCharDataInsert->setEnabled(true);
		m_pActXmlCharDataEdit->setEnabled(false);
		m_pActXmlCharDataDelete->setEnabled(false);

		if ( selectedNode.parentNode().isDocument() )
		{
			m_pActXmlMoveNodeUp->setEnabled(false);
			m_pActXmlMoveNodeDown->setEnabled(false);
		}
		else
		{
			m_pActXmlMoveNodeUp->setEnabled( ! selectedNode.previousSibling().isNull() );
			m_pActXmlMoveNodeDown->setEnabled( ! selectedNode.nextSibling().isNull() );
		}
	}

	// change views
	m_pViewElement->slotChange(selectedNode);

	m_pTabWidget->setTabEnabled( m_pViewElement, true );
	m_pTabWidget->setTabEnabled( m_pViewContents, false );
	m_pTabWidget->setTabEnabled( m_pViewProcInstr, false );

	m_pTabWidget->showPage(m_pViewElement);

	// change path combo
	m_pActPathCombo->insertItem( domTool_getIconForNodeType(selectedNode.nodeType(), false), domTool_getPath(selectedNode) );
}

void KXMLEditorPart::slotSelectionChanged( const QDomCharacterData & selectedNode )
{
	// Enable/Disable actions
	m_pBrowserExt->emit enableAction( "copy", true );

	m_pActEditDeselect->setEnabled(true);
	m_pActViewNodeUp->setEnabled( ! selectedNode.parentNode().isNull() ); // disable if it's a root item
	m_pActViewExpNode->setEnabled(false);
	m_pActViewColNode->setEnabled(false);
	m_pActBookmarksToggle->setEnabled(true);

	if ( isReadWrite() )
	{
		m_pBrowserExt->emit enableAction( "cut", true );
		m_pBrowserExt->emit enableAction( "paste", true );

		m_pActXmlElementInsert->setEnabled(false);
		m_pActXmlElementEdit->setEnabled(false);
		m_pActXmlElementDelete->setEnabled(false);
		m_pActXmlAttributesAdd->setEnabled(false);
		m_pActXmlAttributesDel->setEnabled(false);
		m_pActXmlAttributeDel->setEnabled(false);
		m_pActXmlProcInstrInsert->setEnabled(false);
		m_pActXmlProcInstrEdit->setEnabled(false);
		m_pActXmlProcInstrDelete->setEnabled(false);
		m_pActXmlCharDataInsert->setEnabled(false);
		m_pActXmlCharDataEdit->setEnabled(true);
		m_pActXmlCharDataDelete->setEnabled(true);
		m_pActXmlMoveNodeUp->setEnabled( ! selectedNode.previousSibling().isNull() );
		m_pActXmlMoveNodeDown->setEnabled( ! selectedNode.nextSibling().isNull() );
	}

	// change views
	m_pViewContents->setText( selectedNode.data() );

	m_pTabWidget->setTabEnabled( m_pViewElement, false );
	m_pTabWidget->setTabEnabled( m_pViewContents, true );
	m_pTabWidget->setTabEnabled( m_pViewProcInstr, false );

	m_pTabWidget->showPage( m_pViewContents );

	// change path combo
	m_pActPathCombo->insertItem( domTool_getIconForNodeType(selectedNode.nodeType(), false), domTool_getPath(selectedNode) );
}

void KXMLEditorPart::slotSelectionChanged( const QDomProcessingInstruction & selectedNode )
{
	// Enable/Disable actions
	m_pBrowserExt->emit enableAction( "copy", true );

	m_pActEditDeselect->setEnabled(true);
	m_pActViewNodeUp->setEnabled( ! selectedNode.parentNode().isNull() ); // disable if it's a root item
	m_pActViewExpNode->setEnabled(false);
	m_pActViewColNode->setEnabled(false);
	m_pActBookmarksToggle->setEnabled(true);

	if ( isReadWrite() )
	{
		m_pBrowserExt->emit enableAction( "cut", true );
		m_pBrowserExt->emit enableAction( "paste", true );

		m_pActXmlElementEdit->setEnabled(false);
		m_pActXmlElementDelete->setEnabled(false);
		m_pActXmlAttributesAdd->setEnabled(false);
		m_pActXmlAttributesDel->setEnabled(false);
		m_pActXmlAttributeDel->setEnabled(false);
		m_pActXmlProcInstrInsert->setEnabled(false);
		m_pActXmlProcInstrEdit->setEnabled(true);
		m_pActXmlProcInstrDelete->setEnabled(true);
		m_pActXmlCharDataInsert->setEnabled(false);
		m_pActXmlCharDataEdit->setEnabled(false);
		m_pActXmlCharDataDelete->setEnabled(false);

		if ( selectedNode.parentNode().isDocument() )
		{
			m_pActXmlMoveNodeUp->setEnabled(false);
			m_pActXmlMoveNodeDown->setEnabled(false);
		}
		else
		{
			m_pActXmlMoveNodeUp->setEnabled( ! selectedNode.previousSibling().isNull() );
			m_pActXmlMoveNodeDown->setEnabled( ! selectedNode.nextSibling().isNull() );
		}
	}

	// change views
	m_pViewProcInstr->setText( selectedNode.data() );

	m_pTabWidget->setTabEnabled( m_pViewElement, false );
	m_pTabWidget->setTabEnabled( m_pViewContents, false );
	m_pTabWidget->setTabEnabled( m_pViewProcInstr, true );

	m_pTabWidget->showPage( m_pViewProcInstr );

	// change path combo
	m_pActPathCombo->slotClearEdit();
}

void KXMLEditorPart::slotConfigurationChanged() const
{
	// save the new configuration to our config file
	m_pDlgConfiguration->saveConfiguration( instance()->config() );
}

void KXMLEditorPart::slotContextMenuRequested( const QString & szMenuName, const QPoint & pos )
{
	QWidget * pContainer = hostContainer(szMenuName);

	if ( ! pContainer )
	{
		kdError() << "KXMLEditor " << k_funcinfo << " Couldn't get a container widget for the given menu name (" << szMenuName << ")" << endl;
		return;
	}

	if ( ! pContainer->inherits("KPopupMenu") )
	{
		kdError() << "KXMLEditor " << k_funcinfo << " Wrong container widget" << endl;
		return;
	}

	KPopupMenu * pMenu = static_cast <KPopupMenu*> (pContainer);
	pMenu->popup( pos );
}


/** Copy XML node into clipboard */
QTextDrag * KXMLEditorPart::copyNode(QDomNode * pNode)
{
	QTextDrag *pDrag = 0;
	QString strXML;

	QTextStream streamXML(&strXML, IO_WriteOnly);
	pNode->save(streamXML, 0);
	pDrag = new QTextDrag(strXML, m_pViewTree);

	/*if(pNode->isElement())
		pDrag->setSubtype(KXE_TreeViewItem::m_strSubtypeXML);	

	if(pNode->isProcessingInstruction())
		pDrag->setSubtype(KXE_TreeViewItem::m_strSubtypeXML_procins);		

	if(pNode->isText())
		pDrag->setSubtype(KXE_TreeViewItem::m_strSubtypeXML_text);		

	if(pNode->isCDATASection())
		pDrag->setSubtype(KXE_TreeViewItem::m_strSubtypeXML_cdata);		

	if(pNode->isComment())
		pDrag->setSubtype(KXE_TreeViewItem::m_strSubtypeXML_comment);		*/

	return pDrag;
}

//-----------------------------------------------------------------------------
//
// Paste XML node from clipboard into document
//
// pNode    - target node
// data     - data to pasted
//
//-----------------------------------------------------------------------------
bool KXMLEditorPart::pasteNode(QDomNode * pNode, QMimeSource *data)
{
	QString strText;

	// Drop XML Processing Instruction			
  if(QTextDrag::decode(data, strText))
		{ if(strText.find("<?xml ") == 0)
      {
        KMessageBox::sorry(0, i18n("This processing instruction cannot be pasted here !"));
        return false;
      }

      //---

      QString strXML = "<root>" + strText  + "</root>";

			// Paste clipboard contents as XML element
			QString strErrorMsg;
  		int iErrorLine, iErrorColumn;
			QDomDocument doc;

 		  if(!doc.setContent(strXML, true, &strErrorMsg, &iErrorLine, &iErrorColumn) )
 				{ kdDebug() << "KXMLEditorPart::openFile: Failed parsing the file." << endl;

					KMessageBox::error(m_pViewTree,
												 		i18n("%1 in line %2, column %3").arg(strErrorMsg).arg(iErrorLine).arg(iErrorColumn),
												 		i18n("Parsing error !"));

					return false;
				}

			// check if root item already exists
		  if(!pNode && (m_pDoc->documentElement().isElement()) && (doc.firstChild().firstChild().isElement()))
				{ KMessageBox::sorry(m_pViewTree, i18n("Root element already exists !"));
					return false;
				}

			// Import parsed document to main document
			if(doc.hasChildNodes())
				{ if(pNode == 0)
						{ // Append it as root node
							m_pDoc->importNode(doc.firstChild().firstChild(), true);
							m_pViewTree->updateNodeCreated(m_pDoc->documentElement());
						}
 					else
						{ QDomNode newNode = doc.firstChild().firstChild().cloneNode(true);

              // make some restriction checks for proceessing instructions
              if(pNode->isProcessingInstruction() && !newNode.isProcessingInstruction())
              {
                KMessageBox::sorry(m_pViewTree, i18n("Item cannot be pasted to processing instruction !"));
                return false;
              }
              //---
              							
							if(pNode->isElement())
								{ // Append it as child of pNode
									pNode->appendChild(newNode);
									m_pViewTree->updateNodeCreated(newNode);
								}
              
							if(pNode->isProcessingInstruction() && newNode.isProcessingInstruction())
								{ // Replace contents of selected node
									QDomProcessingInstruction domProcInstr = pNode->toProcessingInstruction();
									QDomProcessingInstruction domProcInstrNew = newNode.toProcessingInstruction();
	
									domProcInstr.setData(domProcInstrNew.data());
									m_pViewProcInstr->setText(domProcInstr.data());
									m_pViewTree->updateNodeChanged(*pNode);
								}

							if(pNode->isCharacterData() && newNode.isCharacterData())
								{ // Replace contents of selected node
									QDomCharacterData domCharacterData = pNode->toCharacterData();
									QDomCharacterData domCharacterDataNew = newNode.toCharacterData();
	
									domCharacterData.setData(domCharacterDataNew.data());
									m_pViewContents->setText(domCharacterDataNew.data());
									m_pViewTree->updateNodeChanged(*pNode);
								}
						}
				}

			setModified();
			return true;
		}
	return false;
}

void KXMLEditorPart::newFile()
{
	switch (m_pDlgConfiguration->getDefaultBehaviour())
	{
		case DlgConfiguration::CreateEmptyFile 	: break;	// nothing to do
		case DlgConfiguration::CreateWithAssistance : slotXmlSpecProcInstrInsert(); break;	// right, we have it defined now
		case DlgConfiguration::UseDefaults 		: QString data("version = '");
										  data += m_pDlgConfiguration->getDefaultVersion()+
										  "' encoding = '"+m_pDlgConfiguration->getDefaultEncoding()+"' ";
										  setSpecProcInstr("xml",data);
										  break;
	}
}

void KXMLEditorPart::setSpecProcInstr(const QString& target, const QString& data)
{
	// removing old one
	removeSpecProcInstr(target);
	// create new one
	QDomProcessingInstruction domProcInstr = m_pDoc->createProcessingInstruction(target,data);

        QDomNode *pNode = m_pViewTree->getSpecProcInstrNode("xml");
	if (pNode)
		// if there is already xml instruction, then put that one below it
		m_pDoc->insertAfter(domProcInstr,*pNode);
	else
		// otherwise put it always on the top
		m_pDoc->insertBefore(domProcInstr,m_pDoc->firstChild());

	m_pViewTree->updateNodeCreated(domProcInstr);
	setModified();
}

void KXMLEditorPart::removeSpecProcInstr(const QString &target)
{
	QDomNode domNode = getSpecProcInstr(target);
	if (!domNode.isNull())
	{
		m_pViewTree->updateNodeDeleted(domNode);
		m_pDoc->removeChild(domNode);
		setModified();
	}
}

void KXMLEditorPart::slotXmlDetachStylesheet()
{
	detachStylesheet();
}

void KXMLEditorPart::slotXmlAttachStylesheet()
{
	DlgXMLAttachBase dlg(widget());
	dlg.Label->setText(i18n("Stylesheet URL:"));
	if (dlg.exec())
	{
		detachStylesheet();
		attachStylesheet(dlg.attachURI->url());
	}
}

void KXMLEditorPart::slotXmlDetachSchema()
{
	detachSchema();
}

void KXMLEditorPart::slotXmlAttachSchema()
{
	DlgXMLAttachBase dlg(widget());
	dlg.Label->setText(i18n("Schema URL:"));
	if (dlg.exec())
	{
		detachSchema();
		attachSchema(dlg.attachURI->url());
	}
}

void KXMLEditorPart::attachStylesheet(const KURL& stylesheet)
{
	setSpecProcInstr("xml-stylesheet",QString("type = 'text/xsl' href = '")+stylesheet.url()+"' ");
}

void KXMLEditorPart::detachStylesheet()
{
	removeSpecProcInstr("xml-stylesheet");
}

void KXMLEditorPart::attachSchema(const KURL& schema)
{
	QDomElement domElement = m_pDoc->documentElement();
	if (!domElement.isNull())
	{
		domElement.setAttributeNS		(SCHEMA_NAMESPACE,
								SCHEMA_ATTRIBUTE_XSI,
								schema.url());
		// refresh view
		QDomNode *pNode = m_pViewTree->getSelectedNode();
		if (pNode!=0)
		{
			QDomElement element = pNode->toElement();
			m_pViewElement->slotChange(element);
		}
	setModified();
	}
}

void KXMLEditorPart::detachSchema()
{
	QDomElement domElement = m_pDoc->documentElement();
	if (!domElement.isNull())
	{
		domElement.removeAttributeNS(SCHEMA_NAMESPACE,SCHEMA_ATTRIBUTE);
		// refresh view
		QDomNode *pNode = m_pViewTree->getSelectedNode();
		if (pNode!=0)
		{
			QDomElement element = pNode->toElement();
			m_pViewElement->slotChange(element);
		}
	setModified();
	}
}

void KXMLEditorPart::updateActions()
{
	m_pActXmlAttachSchema->setEnabled(!m_pDoc->documentElement().isNull());
	m_pActXmlDetachSchema->setEnabled(!m_pDoc->documentElement().isNull() &&
								// that's a question why it works with negated condition ??????
								!m_pDoc->documentElement().hasAttributeNS(SCHEMA_NAMESPACE,SCHEMA_ATTRIBUTE));
	m_pActXmlDetachStylesheet->setEnabled(!getSpecProcInstr("xml-stylesheet").isNull());
}

QDomNode KXMLEditorPart::getSpecProcInstr(const QString& target)
{
	QDomNode result;
	QDomNodeList domNodeList = m_pDoc->childNodes();
	for (uint i=0;i<domNodeList.count();i++)
		if (domNodeList.item(i).isProcessingInstruction())
		{
			QDomProcessingInstruction domProcInstr = domNodeList.item(i).toProcessingInstruction();
			if (domProcInstr.target()==target)
				return domNodeList.item(i);
		}
	return result;
}
