/***************************************************************************
                          kxmleditorpart.h  -  description
                             -------------------
    begin                : Wed Sep 19 2001
    copyright            : (C) 2001, 2002, 2003 by The KXMLEditor Team
    email                : OleBowle@gmx.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef KXMLEDITORPART_H
#define KXMLEDITORPART_H

#include <kparts/part.h>
#include <kparts/browserextension.h>

#include <qdom.h>

#include "kxe_treeview.h"

class QTabWidget;
class KXE_ViewElement;
class DlgSearch;
class DlgConfiguration;
class QTextEdit;
class KAction;
class KToolBarPopupAction;
class KXmlEditorComboAction;
class QDomNode;
class KXMLEditorPartIfaceReadOnly; // DCOP interface

/** This is the KPart of KXMLEditor. */
class KXMLEditorPart : public KParts::ReadWritePart
{
		Q_OBJECT

public:
		/** Creates the actions, the view (widgets) and changes to "begin state". */
		KXMLEditorPart( bool fReadWrite, QWidget * pParent, const char * pszName );
		virtual ~KXMLEditorPart();

		/** Changes the behaviour of this part to readonly or readwrite. */
		virtual void setReadWrite( bool fReadWrite = true );

		/** Returns the selected XML object's path or an empty string, if no object is selected. */
		QString getSelectedPath() const { return m_pViewTree->getSelectedPath(); }

		/**
		 * Reimplemented from @ref KParts::KReadWritePart::setModified.
		 *
		 * Alters the window caption according to the given flag and
		 * calls the base functionality.
		 */
		virtual void setModified( bool fModified );

		/** Copy XML node into clipboard */
		QTextDrag * copyNode(QDomNode *);

		/** Paste XML node from clipboard into document */
		bool pasteNode(QDomNode *, QMimeSource *);

		/**    Prepares content of a newly created XML file. */
		virtual void newFile();

                /** Attaches stylesheet to current XML file. If file is already attached to stylesheet,
		this attachment will be overriten.
		@param stylesheet url to stylesheet location
		@sa detachStylesheet() */
		virtual void attachStylesheet(const KURL& stylesheet);
                /** Detaches stylesheet from current XML file.
		@sa attachStylesheet()*/
		virtual void detachStylesheet();
                /** Attaches schema to current XML file. If file is already attached to schema,
		this attachment will be overriten.
		@param schema url to schema location
		@sa detachSchema()*/
		virtual void attachSchema(const KURL& schema);
                /** Detaches schema from current XML file.
		@sa attachSchema()*/
		virtual void detachSchema();

public slots:

		/**
		 * Reimplemented from @ref KParts::KReadWritePart::setModified.
		 */
		virtual void setModified() { setModified(true); updateActions(); }

		/** Cut the marked text/object into the clipboard */
		void slotEditCut();
		/** Copy the marked text/object into the clipboard */
		void slotEditCopy();
		/** Paste the clipboard into the document */
		void slotEditPaste();
		/** Displays search dialog and finds string in this tree view */
		void slotEditFind();
		/** Finds next occurence of string in this tree view */
		void slotEditFindNext();
		/** Deselects currently selected item */
		void slotEditDeselect() { m_pViewTree->editDeselect(); }
		/** Selects the selected item's parent item. */
		void slotViewNodeUp() { m_pViewTree->viewNodeUp(); }
		/** Expands the selected tree item's subtree. */
		void slotViewExpNode() { m_pViewTree->viewExpNode(-1); }
		/** Expands the selected tree item's subtree to the given level. */
		void slotViewExpNode( int nLevel ) { m_pViewTree->viewExpNode(nLevel); }
		/** Collapses the selected tree item's subtree. */
		void slotViewColNode() { m_pViewTree->viewColNode(0); }
		/** Collapses the selected tree item's subtree to the given level. */
		void slotViewColNode( int nLevel ) { m_pViewTree->viewColNode(nLevel); }
		/** Insert XML element into document */
		void slotXmlElementInsert();
		/** Edit XML element */
		void slotXmlElementEdit();
		/** Delete XML element */
		void slotXmlElementDelete();
		/** Add attribute to an XML element. */
		void slotXmlAttributesAdd();
		/** Delete all attributes of an XML element. */
		void slotXmlAttributesDel();
		/** Delete an attribute of an XML element. */
		void slotXmlAttributeDel();
		/** Insert proc.instruction into document */
		void slotXmlProcInstrInsert();
		/** Edit proc.instruction */
		void slotXmlProcInstrEdit();
		/** Delete proc.instruction */
		void slotXmlProcInstrDelete();
		/** Insert special proc.instruction into document */
		void slotXmlSpecProcInstrInsert();
		/* Insert character data */
		void slotXmlCharDataInsert();
		/* Edit character data */
		void slotXmlCharDataEdit();
		/* Delete character data */
		void slotXmlCharDataDelete();
		/* Moves a node up (change its position with its prev. siblings position) */
		void slotXmlMoveNodeUp();
		/* Moves a node down (change its position with its next siblings position) */
		void slotXmlMoveNodeDown();
		/** toggles bookmark on the selected item */
		void slotBookmarksToggle();
		/** searches for the previous bookmarked item */
		void slotBookmarksPrev() { m_pViewTree->bookmarksPrev(); }
		/** searches for the next bookmarked item */
		void slotBookmarksNext() { m_pViewTree->bookmarksNext(); }
		/** shows configuration dialog (@ref OptionsDialog) */
		void slotConfigure();

		void slotPathSelected( const QString & szPath );
		void slotPathClear();
		/**
		 * Called when a tree view items text was changed via inplace-renaming.
		 */
		void slotItemRenamedInplace( QListViewItem * pItem );
		/** Called whan an attributes value was changed via inplace editing. */
		void slotAttributeChangedInplace( const QDomAttr & );

		/** Called on user's action response. Creates <?xml-stylesheet ...?> processing instruction
		with reference to specified stylesheet URI.
		*/
		void slotXmlAttachStylesheet();

		/** Called on user's action response. */
		void slotXmlDetachStylesheet();

		/** Called on user's action response */
		void slotXmlDetachSchema();

		/** Called on user's action response */
		void slotXmlAttachSchema();

protected:
		/** Opens the file @ref KParts::ReadOnlyPart::m_file. */
		virtual bool openFile();
		/** Saves the file @ref KParts::ReadOnlyPart::m_file. */
		virtual bool saveFile();
		/** Adds or changes (if already exists) processing instruction. Processing
		instruction will be added as a child in QDomDocument object.
			@param target processing intruction target
			@param data processing intruction data */
		void setSpecProcInstr(const QString& target, const QString& data);
		/** Removes spec. processing instruction from XML file.
			@param target name of the proc. instruction target
			@warning Removes processing instruction from DomDocument children node list only (no subnodes are modiified).
		*/
		void removeSpecProcInstr(const QString &target);
		/** Returns processing instruction located in QDomDocument. If specified
		processing intruction is not found, null node is returned.
			@param target name of the processing instruction target. */
		QDomNode getSpecProcInstr(const QString& target);
		/** Updates status of actions (enabled/disabled). Is it automatically invoked everytime content is modified.
		Good place for updating actions which do not depend on current selection, but are file content dependent,
		like #m_pActXmlAttachStylesheet #m_pActXmlAttachSchema.
			@sa setModified()
		*/
		void updateActions();

		// the actions
		KAction * m_pActEditFind;
		KAction * m_pActEditFindNext;
		KAction * m_pActEditDeselect;
		KAction * m_pActViewNodeUp;
		KToolBarPopupAction * m_pActViewExpNode;
		KToolBarPopupAction * m_pActViewColNode;
		KAction * m_pActXmlElementInsert;
		KAction * m_pActXmlElementEdit;
		KAction * m_pActXmlElementDelete;
		KAction * m_pActXmlAttributesAdd;
		KAction * m_pActXmlAttributesDel;
		KAction * m_pActXmlAttributeDel;
		KAction * m_pActXmlProcInstrInsert;
		KAction * m_pActXmlProcInstrEdit;
		KAction * m_pActXmlProcInstrDelete;
		KAction * m_pActXmlSpecProcInstrInsert;
		KAction * m_pActXmlCharDataInsert;
		KAction * m_pActXmlCharDataEdit;
		KAction * m_pActXmlCharDataDelete;
		KAction * m_pActXmlMoveNodeUp;
		KAction * m_pActXmlMoveNodeDown;
		KAction * m_pActBookmarksToggle;
		KAction * m_pActBookmarksPrev;
		KAction * m_pActBookmarksNext;
		KXmlEditorComboAction * m_pActPathCombo;

		/** the document */
		QDomDocument * m_pDoc;

	protected slots:
		void started();
		void completed();
		void canceled();
		/** enables/disables actions and changes views  */
		void slotSelectionCleared(bool);
		/** enables/disables actions depending on the given XML element and changes views  */
		void slotSelectionChanged( const QDomElement & );
		/** enables/disables actions and changes views */
		void slotSelectionChanged( const QDomCharacterData & );
		/** enables/disables actions and changes views */
		void slotSelectionChanged( const QDomProcessingInstruction & );
		/** saves the configuration to the config file */
		void slotConfigurationChanged() const;

		/** Shows the requested context menu at the given position. */
		void slotContextMenuRequested( const QString & szMenuName, const QPoint & pos );

	private:
		/** this view displays the XML-document's tree structure */
		KXE_TreeView * m_pViewTree;
		/** the tab widget, contains: m_pViewElement, m_pViewContents */
		QTabWidget * m_pTabWidget;
		/** widget to display XML element */
		KXE_ViewElement * m_pViewElement;
		/** edit widget for contents */
		QTextEdit * m_pViewContents;
		/** widget to display a proc.instr. */
		QTextEdit * m_pViewProcInstr;
		/** the search dialog, used for find action */
		DlgSearch * m_pDlgSearch;
		/** the configuration dialog */
		DlgConfiguration * m_pDlgConfiguration;

	protected:
		/** DCOP object having access to XML document. It may by NULL if no DCOP inteface is provided
		* or read only privided by @ref KXMLEditorPartReadOnlyIface or full read-write @ref KXMLEditorPartIface .
		*/
		KXMLEditorPartIfaceReadOnly * m_pDCOPIface;

		/** our browser extension */
		KParts::BrowserExtension * m_pBrowserExt;

		// Indicates, if document is opened from tar.gz
		bool m_bDocIsCompressed;

		/** Name of entry in Tar archive */
		QString m_strCompressedTarEntryName;

		/** Action for attaching stylehseets to xml files. */
		KAction* m_pActXmlAttachStylesheet;
		/** Action for detaching stylehseets from xml files. */
		KAction* m_pActXmlDetachStylesheet;
		/** Action for attaching schemas to xml files. */
		KAction* m_pActXmlAttachSchema;
		/** Action for detaching schemas from xml files. */
		KAction* m_pActXmlDetachSchema;
};



/**
 * The browser extension for our part (@ref KXMLEditorPart).
 */
class KXMLEditorBrowserExtension : public KParts::BrowserExtension
{
		Q_OBJECT

	public:
		KXMLEditorBrowserExtension( KXMLEditorPart * pParent, const char * pszName = "KXMLEditorBrowserExtension" ) : KParts::BrowserExtension( pParent, pszName ), m_pPart(pParent) {}

	public slots:
		void cut() { m_pPart->slotEditCut(); }
		void copy() { m_pPart->slotEditCopy(); }
		void paste() { m_pPart->slotEditPaste(); }

	protected:
		KXMLEditorPart * m_pPart;
};

#endif

